@@ -46,8 +46,7 @@
 #include <unistd.h>
 #endif
 
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-lowlevel.h>
+#include <gio/gio.h>
 
 #include <xfconf/xfconf.h>
 
@@ -70,6 +69,8 @@
 
 static gboolean opt_disable_tcp = FALSE;
 static gboolean opt_version = FALSE;
+static XfconfChannel *channel = NULL;
+static guint name_id = 0;
 
 static GOptionEntry option_entries[] =
 {
@@ -78,6 +79,12 @@ static GOptionEntry option_entries[] =
   { NULL }
 };
 
+
+static void name_lost (GDBusConnection *connection,
+                       const gchar *name,
+                       gpointer user_data);
+
+
 static void
 setup_environment (void)
 {
@@ -128,7 +135,6 @@ setup_environment (void)
 static void
 init_display (XfsmManager   *manager,
               GdkDisplay    *dpy,
-              XfconfChannel *channel,
               gboolean       disable_tcp)
 {
   gchar *engine;
@@ -142,55 +148,139 @@ init_display (XfsmManager   *manager,
   gdk_flush ();
 
   sm_init (channel, disable_tcp, manager);
-
-  /* gtk resource files may have changed */
-  gtk_rc_reparse_all ();
 }
 
 
+
 static void
-xfsm_dbus_init (void)
+manager_quit_cb (XfsmManager *manager, gpointer user_data)
 {
-  DBusGConnection *dbus_conn;
-  int              ret;
-  GError          *error = NULL;
+  GDBusConnection *connection = G_DBUS_CONNECTION (user_data);
 
-  xfsm_error_dbus_init ();
+  g_bus_unown_name (name_id);
 
-  dbus_conn = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-  if (G_UNLIKELY (!dbus_conn))
+  name_lost (connection, "xfce4-session", manager);
+}
+
+
+
+static void
+bus_acquired (GDBusConnection *connection,
+              const gchar *name,
+              gpointer user_data)
+{
+  XfsmManager      *manager = XFSM_MANAGER (user_data);
+  GdkDisplay       *dpy;
+
+  xfsm_verbose ("bus_acquired %s\n", name);
+
+  manager = xfsm_manager_new (connection);
+
+  if (manager == NULL) {
+          g_critical ("Could not create XfsmManager");
+  }
+
+  g_signal_connect(G_OBJECT(manager),
+                   "manager-quit",
+                   G_CALLBACK(manager_quit_cb),
+                   connection);
+
+  setup_environment ();
+
+  channel = xfsm_open_config ();
+
+  dpy = gdk_display_get_default ();
+  init_display (manager, dpy, opt_disable_tcp);
+
+  if (!opt_disable_tcp && xfconf_channel_get_bool (channel, "/security/EnableTcp", FALSE))
     {
-      g_critical ("Unable to contact D-Bus session bus: %s", error ? error->message : "Unknown error");
-      if (error)
-        g_error_free (error);
-      return;
+      /* verify that the DNS settings are ok */
+      xfsm_splash_screen_next (splash_screen, _("Verifying DNS settings"));
+      xfsm_dns_check ();
     }
 
-  ret = dbus_bus_request_name (dbus_g_connection_get_connection (dbus_conn),
-                               "org.xfce.SessionManager",
-                               DBUS_NAME_FLAG_DO_NOT_QUEUE,
-                               NULL);
-  if (DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER != ret)
+  xfsm_splash_screen_next (splash_screen, _("Loading session data"));
+
+  xfsm_startup_init (channel);
+  xfsm_manager_load (manager, channel);
+  xfsm_manager_restart (manager);
+}
+
+
+
+static void
+name_acquired (GDBusConnection *connection,
+               const gchar *name,
+               gpointer user_data)
+{
+  xfsm_verbose ("name_acquired\n");
+}
+
+
+
+static void
+name_lost (GDBusConnection *connection,
+           const gchar *name,
+           gpointer user_data)
+{
+  XfsmManager      *manager = XFSM_MANAGER (user_data);
+  GError           *error = NULL;
+  XfsmShutdownType  shutdown_type;
+  XfsmShutdown     *shutdown_helper;
+  gboolean          succeed = TRUE;
+
+  xfsm_verbose ("name_lost\n");
+
+  /* Release the  object */
+  xfsm_verbose ("Disconnected from D-Bus");
+
+  shutdown_type = xfsm_manager_get_shutdown_type (manager);
+
+  /* take over the ref before we release the manager */
+  shutdown_helper = xfsm_shutdown_get ();
+
+  ice_cleanup ();
+
+  if (shutdown_type == XFSM_SHUTDOWN_SHUTDOWN
+      || shutdown_type == XFSM_SHUTDOWN_RESTART)
+    {
+      succeed = xfsm_shutdown_try_type (shutdown_helper, shutdown_type, &error);
+      if (!succeed)
+        g_warning ("Failed to shutdown/restart: %s", ERROR_MSG (error));
+    }
+
+  g_object_unref (shutdown_helper);
+  g_object_unref (manager);
+  g_object_unref (channel);
+  g_clear_error (&error);
+
+  shutdown_helper = NULL;
+  manager = NULL;
+  channel = NULL;
+
+  gtk_main_quit ();
+}
+
+
+
+static void
+xfsm_dbus_init (XfsmManager *manager)
+{
+  name_id = g_bus_own_name (G_BUS_TYPE_SESSION,
+                            "org.xfce.SessionManager",
+                            G_BUS_NAME_OWNER_FLAGS_NONE,
+                            bus_acquired, name_acquired, name_lost,
+                            manager,
+                            NULL);
+
+  if (name_id == 0)
     {
       g_printerr ("%s: Another session manager is already running\n", PACKAGE_NAME);
       exit (EXIT_FAILURE);
     }
 }
 
-static void
-xfsm_dbus_cleanup (void)
-{
-  DBusGConnection *dbus_conn;
 
-  /* this is all not really necessary, but... */
-
-  dbus_conn = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
-  if (G_UNLIKELY (!dbus_conn))
-    return;
-
-  dbus_bus_release_name (dbus_g_connection_get_connection (dbus_conn),
-                         "org.xfce.SessionManager", NULL);
-}
 
 static gboolean
 xfsm_dbus_require_session (gint argc, gchar **argv)
@@ -235,12 +325,8 @@ xfsm_dbus_require_session (gint argc, gchar **argv)
 int
 main (int argc, char **argv)
 {
-  XfsmManager      *manager;
+  XfsmManager      *manager = NULL;
   GError           *error = NULL;
-  GdkDisplay       *dpy;
-  XfconfChannel    *channel;
-  XfsmShutdownType  shutdown_type;
-  XfsmShutdown     *shutdown_helper;
   gboolean          succeed = TRUE;
 
   if (!xfsm_dbus_require_session (argc, argv))
@@ -263,7 +349,7 @@ main (int argc, char **argv)
   if (opt_version)
     {
       g_print ("%s %s (Xfce %s)\n\n", G_LOG_DOMAIN, PACKAGE_VERSION, xfce_version_string ());
-      g_print ("%s\n", "Copyright (c) 2003-2014");
+      g_print ("%s\n", "Copyright (c) 2003-2016");
       g_print ("\t%s\n\n", _("The Xfce development team. All rights reserved."));
       g_print (_("Please report bugs to <%s>."), PACKAGE_BUGREPORT);
       g_print ("\n");
@@ -280,55 +366,11 @@ main (int argc, char **argv)
   /* fake a client id for the manager, so the legacy management does not
    * recognize us to be a session client.
    */
-  gdk_set_sm_client_id (xfsm_generate_client_id (NULL));
+  gdk_x11_set_sm_client_id (xfsm_generate_client_id (NULL));
 
-  xfsm_dbus_init ();
-
-  manager = xfsm_manager_new ();
-  setup_environment ();
-
-  channel = xfsm_open_config ();
-
-  dpy = gdk_display_get_default ();
-  init_display (manager, dpy, channel, opt_disable_tcp);
-
-  if (!opt_disable_tcp && xfconf_channel_get_bool (channel, "/security/EnableTcp", FALSE))
-    {
-      /* verify that the DNS settings are ok */
-      xfsm_splash_screen_next (splash_screen, _("Verifying DNS settings"));
-      xfsm_dns_check ();
-    }
-
-  xfsm_splash_screen_next (splash_screen, _("Loading session data"));
-
-  xfsm_startup_init (channel);
-  xfsm_manager_load (manager, channel);
-  xfsm_manager_restart (manager);
+  xfsm_dbus_init (manager);
 
   gtk_main ();
 
-  xfsm_startup_shutdown ();
-
-  shutdown_type = xfsm_manager_get_shutdown_type (manager);
-
-  /* take over the ref before we release the manager */
-  shutdown_helper = xfsm_shutdown_get ();
-
-  g_object_unref (manager);
-  g_object_unref (channel);
-
-  xfsm_dbus_cleanup ();
-  ice_cleanup ();
-
-  if (shutdown_type == XFSM_SHUTDOWN_SHUTDOWN
-      || shutdown_type == XFSM_SHUTDOWN_RESTART)
-    {
-      succeed = xfsm_shutdown_try_type (shutdown_helper, shutdown_type, &error);
-      if (!succeed)
-        g_warning ("Failed to shutdown/restart: %s", ERROR_MSG (error));
-    }
-
-  g_object_unref (shutdown_helper);
-
   return succeed ? EXIT_SUCCESS : EXIT_FAILURE;
 }
