@@ -71,7 +71,7 @@ typedef struct _XfsmManager  XfsmManager;
 
 GType xfsm_manager_get_type (void) G_GNUC_CONST;
 
-XfsmManager *xfsm_manager_new (void);
+XfsmManager *xfsm_manager_new (GDBusConnection *connection);
 
 void xfsm_manager_load (XfsmManager   *manager,
                         XfconfChannel *channel);
@@ -91,6 +91,7 @@ XfsmClient* xfsm_manager_new_client (XfsmManager *manager,
 
 gboolean xfsm_manager_register_client (XfsmManager *manager,
                                        XfsmClient  *client,
+                                       const gchar *dbus_client_id,
                                        const gchar *previous_id);
 
 void xfsm_manager_start_interact (XfsmManager *manager,
